/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2024, Qualcomm Innovation Center, Inc. All rights reserved.
 */

#ifndef _DT_BINDINGS_CLK_QCOM_TCSR_CC_SM8750_H
#define _DT_BINDINGS_CLK_QCOM_TCSR_CC_SM8750_H

/* TCSR_CC clocks */
#define TCSR_PCIE_0_CLKREF_EN				0
#define TCSR_UFS_CLKREF_EN				1
#define TCSR_USB2_CLKREF_EN				2
#define TCSR_USB3_CLKREF_EN				3

#endif
