/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
    cs5345.h - definition for cs5345 inputs and outputs

    Copyright (C) 2007 Hans Verkuil (hverkuil@kernel.org)

*/

#ifndef _CS5345_H_
#define _CS5345_H_

/* CS5345 HW inputs */
#define CS5345_IN_MIC 0
#define CS5345_IN_1   1
#define CS5345_IN_2   2
#define CS5345_IN_3   3
#define CS5345_IN_4   4
#define CS5345_IN_5   5
#define CS5345_IN_6   6

#define CS5345_MCLK_1   0x00
#define CS5345_MCLK_1_5 0x10
#define CS5345_MCLK_2   0x20
#define CS5345_MCLK_3   0x30
#define CS5345_MCLK_4   0x40

#endif
