/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Definitions for the Intel integrated DMA 64-bit
 *
 * Copyright (C) 2019 Intel Corporation
 */

#ifndef __LINUX_DMA_IDMA64_H__
#define __LINUX_DMA_IDMA64_H__

/* Platform driver name */
#define LPSS_IDMA64_DRIVER_NAME		"idma64"

#endif /* __LINUX_DMA_IDMA64_H__ */
